@@ -32,7 +32,9 @@ import inspect
 import txaio
 txaio.use_twisted()  # noqa
 
-from twisted.internet.defer import inlineCallbacks
+from twisted.internet.defer import inlineCallbacks, succeed
+
+from autobahn.util import public
 
 from autobahn.websocket.util import parse_url as parse_ws_url
 from autobahn.rawsocket.util import parse_url as parse_rs_url
@@ -65,11 +67,19 @@ except (ImportError, SyntaxError):
     __all__.pop(__all__.index('Service'))
 
 
+@public
 class ApplicationSession(protocol.ApplicationSession):
     """
     WAMP application session for Twisted-based applications.
+
+    Implements:
+
+        * :class:`autobahn.wamp.interfaces.ITransportHandler`
+        * :class:`autobahn.wamp.interfaces.ISession`
     """
 
+    log = txaio.make_logger()
+
 
 class ApplicationSessionFactory(protocol.ApplicationSessionFactory):
     """
@@ -81,7 +91,10 @@ class ApplicationSessionFactory(protocol.ApplicationSessionFactory):
     The application session class this application session factory will use. Defaults to :class:`autobahn.twisted.wamp.ApplicationSession`.
     """
 
+    log = txaio.make_logger()
 
+
+@public
 class ApplicationRunner(object):
     """
     This class is a convenience tool mainly for development and quick hosting
@@ -97,10 +110,10 @@ class ApplicationRunner(object):
         """
 
         :param url: The WebSocket URL of the WAMP router to connect to (e.g. `ws://somehost.com:8090/somepath`)
-        :type url: unicode
+        :type url: str
 
         :param realm: The WAMP realm to join the application session to.
-        :type realm: unicode
+        :type realm: str
 
         :param extra: Optional extra configuration to forward to the application component.
         :type extra: dict
@@ -133,15 +146,28 @@ class ApplicationRunner(object):
         self.proxy = proxy
         self.headers = headers
 
+        # this if for auto-reconnection when Twisted ClientService is avail
+        self._client_service = None
+        # total number of successful connections
+        self._connect_successes = 0
+
+    def stop(self):
+        self.log.debug('{klass}.stop()', klass=self.__class__.__name__)
+        if self._client_service:
+            return self._client_service.stopService()
+        else:
+            return succeed(None)
+
+    @public
     def run(self, make, start_reactor=True, auto_reconnect=False, log_level='info'):
         """
         Run the application component.
 
-        :param make: A factory that produces instances of :class:`autobahn.asyncio.wamp.ApplicationSession`
+        :param make: A factory that produces instances of :class:`autobahn.twisted.wamp.ApplicationSession`
            when called with an instance of :class:`autobahn.wamp.types.ComponentConfig`.
         :type make: callable
 
-        :param start_reactor: if True (the default) this method starts
+        :param start_reactor: When ``True`` (the default) this method starts
            the Twisted reactor and doesn't return until the reactor
            stops. If there are any problems starting the reactor or
            connect()-ing, we stop the reactor and raise the exception
@@ -261,6 +287,7 @@ class ApplicationRunner(object):
         # when our proto was created and connected, make sure it's cleaned
         # up properly later on when the reactor shuts down for whatever reason
         def init_proto(proto):
+            self._connect_successes += 1
             reactor.addSystemEventTrigger('before', 'shutdown', cleanup, proto)
             return proto
 
@@ -269,19 +296,38 @@ class ApplicationRunner(object):
             try:
                 # since Twisted 16.1.0
                 from twisted.application.internet import ClientService
+                from twisted.application.internet import backoffPolicy
                 use_service = True
             except ImportError:
                 use_service = False
 
         if use_service:
+            # this code path is automatically reconnecting ..
             self.log.debug('using t.a.i.ClientService')
-            # this is automatically reconnecting
-            service = ClientService(client, transport_factory)
-            service.startService()
-            d = service.whenConnected()
+
+            default_retry = backoffPolicy()
+
+            if False:
+                # retry policy that will only try to reconnect if we connected
+                # successfully at least once before (so it fails on host unreachable etc ..)
+                def retry(failed_attempts):
+                    if self._connect_successes > 0:
+                        return default_retry(failed_attempts)
+                    else:
+                        self.stop()
+                        return 100000000000000
+            else:
+                retry = default_retry
+
+            self._client_service = ClientService(client, transport_factory, retryPolicy=retry)
+            self._client_service.startService()
+
+            d = self._client_service.whenConnected()
+
         else:
-            # this is only connecting once!
+            # this code path is only connecting once!
             self.log.debug('using t.i.e.connect()')
+
             d = client.connect(transport_factory)
 
         # if we connect successfully, the arg is a WampWebSocketClientProtocol
